/* Generated automatically from crt.inc
   DO NOT CHANGE THIS FILE MANUALLY! */

#define ksUnknown     255
#define ksLeft        75
#define ksRight       77
#define ksUp          72
#define ksDown        80
#define ksPgUp        73
#define ksPgDn        81
#define ksHome        71
#define ksEnd         79
#define ksIns         82
#define ksDel         83
#define ksCenter      76
#define ksShTab       15
#define ksShIns       246
#define ksShDel       247
#define ksCtrlTab     148
#define ksCtrlLeft    115
#define ksCtrlRight   116
#define ksCtrlUp      141
#define ksCtrlDown    145
#define ksCtrlPgUp    132
#define ksCtrlPgDn    118
#define ksCtrlHome    119
#define ksCtrlEnd     117
#define ksCtrlIns     146
#define ksCtrlDel     147
#define ksCtrlBkSp    167
#define ksCtrlCentr   143
#define ksShCtrlIns   206
#define ksShCtrlDel   207
#define ksAltTab      165
#define ksAltLeft     155
#define ksAltRight    157
#define ksAltUp       152
#define ksAltDown     160
#define ksAltPgUp     153
#define ksAltPgDn     161
#define ksAltHome     151
#define ksAltEnd      159
#define ksAltIns      162
#define ksAltDel      163
#define ksAltEnter    166
#define ksAltPStar    55
#define ksAltPMinus   74
#define ksAltPPlus    78
#define ksAltEsc      1
#define ksAltSpace    2
#define ksAltBkSp     14
#define ksAltMinus    130
#define ksAltEqual    131
#define ksAltLBrack   26
#define ksAltRBrack   27
#define ksAltSemic    39
#define ksAltFQuote   40
#define ksAltBQuote   41
#define ksAltComma    51
#define ksAltStop     52
#define ksAltFSlash   53
#define ksAltBSlash   43
#define ksAlt0        129
#define ksAlt1        120
#define ksAlt2        121
#define ksAlt3        122
#define ksAlt4        123
#define ksAlt5        124
#define ksAlt6        125
#define ksAlt7        126
#define ksAlt8        127
#define ksAlt9        128
#define ksAltA        30
#define ksAltB        48
#define ksAltC        46
#define ksAltD        32
#define ksAltE        18
#define ksAltF        33
#define ksAltG        34
#define ksAltH        35
#define ksAltI        23
#define ksAltJ        36
#define ksAltK        37
#define ksAltL        38
#define ksAltM        50
#define ksAltN        49
#define ksAltO        24
#define ksAltP        25
#define ksAltQ        16
#define ksAltR        19
#define ksAltS        31
#define ksAltT        20
#define ksAltU        22
#define ksAltV        47
#define ksAltW        17
#define ksAltX        45
#define ksAltY        21
#define ksAltZ        44
#define ksAltGr0      168
#define ksAltGr1      169
#define ksAltGr2      170
#define ksAltGr3      171
#define ksAltGr4      172
#define ksAltGr5      173
#define ksAltGr6      174
#define ksAltGr7      175
#define ksAltGr8      176
#define ksAltGr9      177
#define ksAltGrA      178
#define ksAltGrB      179
#define ksAltGrC      180
#define ksAltGrD      181
#define ksAltGrE      182
#define ksAltGrF      183
#define ksAltGrG      184
#define ksAltGrH      185
#define ksAltGrI      186
#define ksAltGrJ      187
#define ksAltGrK      188
#define ksAltGrL      189
#define ksAltGrM      190
#define ksAltGrN      191
#define ksAltGrO      192
#define ksAltGrP      193
#define ksAltGrQ      194
#define ksAltGrR      195
#define ksAltGrS      196
#define ksAltGrT      197
#define ksAltGrU      198
#define ksAltGrV      199
#define ksAltGrW      200
#define ksAltGrX      201
#define ksAltGrY      202
#define ksAltGrZ      203
#define ksExtra0      208
#define ksExtra1      209
#define ksExtra2      210
#define ksExtra3      211
#define ksExtra4      212
#define ksExtra5      213
#define ksExtra6      214
#define ksExtra7      215
#define ksExtra8      216
#define ksExtra9      217
#define ksExtraA      218
#define ksExtraB      219
#define ksExtraC      220
#define ksExtraD      221
#define ksExtraE      222
#define ksExtraF      223
#define ksExtraG      224
#define ksExtraH      225
#define ksExtraI      226
#define ksExtraJ      227
#define ksExtraK      228
#define ksExtraL      229
#define ksExtraM      230
#define ksExtraN      231
#define ksExtraO      232
#define ksExtraP      233
#define ksExtraQ      234
#define ksExtraR      235
#define ksExtraS      236
#define ksExtraT      237
#define ksExtraU      238
#define ksExtraV      239
#define ksExtraW      240
#define ksExtraX      241
#define ksExtraY      242
#define ksExtraZ      243
#define ksF1          59
#define ksF2          60
#define ksF3          61
#define ksF4          62
#define ksF5          63
#define ksF6          64
#define ksF7          65
#define ksF8          66
#define ksF9          67
#define ksF10         68
#define ksF11         133
#define ksF12         134
#define ksShF1        84
#define ksShF2        85
#define ksShF3        86
#define ksShF4        87
#define ksShF5        88
#define ksShF6        89
#define ksShF7        90
#define ksShF8        91
#define ksShF9        92
#define ksShF10       93
#define ksShF11       135
#define ksShF12       136
#define ksCtrlF1      94
#define ksCtrlF2      95
#define ksCtrlF3      96
#define ksCtrlF4      97
#define ksCtrlF5      98
#define ksCtrlF6      99
#define ksCtrlF7      100
#define ksCtrlF8      101
#define ksCtrlF9      102
#define ksCtrlF10     103
#define ksCtrlF11     137
#define ksCtrlF12     138
#define ksAltF1       104
#define ksAltF2       105
#define ksAltF3       106
#define ksAltF4       107
#define ksAltF5       108
#define ksAltF6       109
#define ksAltF7       110
#define ksAltF8       111
#define ksAltF9       112
#define ksAltF10      113
#define ksAltF11      139
#define ksAltF12      140
#define ksCancel      3
#define ksCopy        4
#define ksUndo        5
#define ksRedo        6
#define ksOpen        7
#define ksClose       8
#define ksCommand     9
#define ksCreate      10
#define ksExit        11
#define ksFind        12
#define ksHelp        13
#define ksMark        28
#define ksMessage     29
#define ksMove        42
#define ksNext        54
#define ksPrevious    56
#define ksOptions     57
#define ksReference   58
#define ksRefresh     69
#define ksReplace     70
#define ksResize      114
#define ksRestart     142
#define ksSuspend     144
#define ksResume      149
#define ksSave        150
#define ksMenu        154
#define ksClear       156
#define ksSelect      158
#define ksStop        164

#define ksInt                250
#define ksTerm               251
#define ksHUp                252
#define ksError              253
#define ksScreenSizeChanged  254

#define chBell        7
#define chBkSp        8
#define chTab         9
#define chLF          10
#define chFF          12
#define chCR          13
#define chEnter       chCR
#define chEsc         27
#define chSpace       ' '
#define chNBSp        160

#define chCtrlA       1
#define chCtrlB       2
#define chCtrlC       3
#define chCtrlD       4
#define chCtrlE       5
#define chCtrlF       6
#define chCtrlG       7
#define chCtrlH       8
#define chCtrlI       9
#define chCtrlJ       10
#define chCtrlK       11
#define chCtrlL       12
#define chCtrlM       13
#define chCtrlN       14
#define chCtrlO       15
#define chCtrlP       16
#define chCtrlQ       17
#define chCtrlR       18
#define chCtrlS       19
#define chCtrlT       20
#define chCtrlU       21
#define chCtrlV       22
#define chCtrlW       23
#define chCtrlX       24
#define chCtrlY       25
#define chCtrlZ       26

#define chLineHS      196
#define chLineVS      179
#define chLineHD      205
#define chLineVD      186
#define chLineHSS     chLineHS
#define chLineVSS     chLineVS
#define chLineHDS     chLineHD
#define chLineVDS     chLineVS
#define chLineHSD     chLineHS
#define chLineVSD     chLineVD
#define chLineHDD     chLineHD
#define chLineVDD     chLineVD
#define chCornerTLS   218
#define chCornerTRS   191
#define chCornerBLS   192
#define chCornerBRS   217
#define chCornerTLD   201
#define chCornerTRD   187
#define chCornerBLD   200
#define chCornerBRD   188
#define chCornerTLSS  chCornerTLS
#define chCornerTRSS  chCornerTRS
#define chCornerBLSS  chCornerBLS
#define chCornerBRSS  chCornerBRS
#define chCornerTLDS  213
#define chCornerTRDS  184
#define chCornerBLDS  212
#define chCornerBRDS  190
#define chCornerTLSD  214
#define chCornerTRSD  183
#define chCornerBLSD  211
#define chCornerBRSD  189
#define chCornerTLDD  chCornerTLD
#define chCornerTRDD  chCornerTRD
#define chCornerBLDD  chCornerBLD
#define chCornerBRDD  chCornerBRD
#define chTeeTS       194
#define chTeeBS       193
#define chTeeLS       195
#define chTeeRS       180
#define chTeeTD       203
#define chTeeBD       202
#define chTeeLD       204
#define chTeeRD       185
#define chTeeTSS      chTeeTS
#define chTeeBSS      chTeeBS
#define chTeeLSS      chTeeLS
#define chTeeRSS      chTeeRS
#define chTeeTDS      209
#define chTeeBDS      207
#define chTeeLDS      198
#define chTeeRDS      181
#define chTeeTSD      210
#define chTeeBSD      208
#define chTeeLSD      199
#define chTeeRSD      182
#define chTeeTDD      chTeeTD
#define chTeeBDD      chTeeBD
#define chTeeLDD      chTeeLD
#define chTeeRDD      chTeeRD
#define chCenterS     197
#define chCenterD     206
#define chCenterSS    chCenterS
#define chCenterDS    216
#define chCenterSD    215
#define chCenterDD    chCenterD

#define chArrowU      24
#define chArrowD      25
#define chArrowL      27
#define chArrowR      26
#define chArrowUD     18
#define chArrowLR     29
#define chTriangleU   30
#define chTriangleD   31
#define chTriangleL   17
#define chTriangleR   16
#define chBlock       219
#define chBlockT      223
#define chBlockB      220
#define chBlockL      221
#define chBlockR      222

#define chClubs       5
#define chSpades      6
#define chHearts      3
#define chDiamonds    4
#define chChkBoard1   176
#define chChkBoard2   177
#define chChkBoard3   178
#define chDegree      248
#define chPlusMinus   241
#define chBullet      254
#define chLEqual      243
#define chGEqual      242
#define chPi          227
#define chSmilie1     1
#define chSmilie2     2
#define chParagraph   21

#define shLeftShift   1
#define shRightShift  2
#define shShift       (shRightShift + shLeftShift)
#define shLeftCtrl    4
#define shRightCtrl   8
#define shCtrl        (shRightCtrl + shLeftCtrl)
#define shLeftAlt     16
#define shAlt         shLeftAlt
#define shRightAlt    32
#define shAltGr       shRightAlt
#define shAnyAlt      (shLeftAlt + shRightAlt)
#define shExtra       64

#if defined (X11) && !defined (XCURSES)
#define XCURSES
#endif
#if defined (XCURSES) && !defined (X11)
#define X11
#endif
#if !defined (USE_PDCURSES) && !defined (USE_NCURSES)
#if defined (MSDOS) || defined (__MINGW32__) || defined (__CYGWIN__) || defined (__EMX__) || defined (XCURSES)
#define USE_PDCURSES
#else
#define USE_NCURSES
#endif
#endif
