{ rts-config.inc.  Generated automatically from rts-config.h.  }
{ rts-config.h.  Generated by configure.  }
{ rts-config.in.  Generated from configure.in by autoheader.  }
{ acconfig.h.  Generated automatically from configure.in using make-acconfig-h.m4.  }

{ Define if struct passwd in <pwd.h> contains a pw_gecos field.  }
{$define HAVE_PW_GECOS 1}

{ Define if struct passwd in <pwd.h> contains a pw_passwd field.  }
{$define HAVE_PW_PASSWD 1}

{ Define if struct utsname in <sys/utsname.h> contains a domainname field.  }
{ #undef HAVE_DOMAINNAME }

{ Define if struct tm in <time.h> contains a tm_isdst field.  }
{$define HAVE_TM_ISDST 1}

{ Define if struct tm in <time.h> contains a tm_gmtoff field.  }
{$define HAVE_TM_GMTOFF 1}

{ Define if type ssize_t is declared in <stdlib.h> or <sys/types.h>.  }
{$define HAVE_SSIZE_T 1}

{ Define if type off_t is declared in <stdlib.h> or <sys/types.h>.  }
{$define HAVE_OFF_T 1}

{ Define if type off64_t is declared in <stdlib.h> or <sys/types.h>.  }
{ #undef HAVE_OFF64_T }

{ Define if type __off64_t is declared in <stdlib.h> or <sys/types.h>.  }
{ #undef HAVE___OFF64_T }

{ Define if type loff_t is declared in <stdlib.h> or <sys/types.h>.  }
{ #undef HAVE_LOFF_T }

{ Define if type offset_t is declared in <stdlib.h> or <sys/types.h>.  }
{ #undef HAVE_OFFSET_T }

{ Define if type struct stat64 is declared in <sys/stat.h>.  }
{ #undef HAVE_STRUCT_STAT64 }

{ Define if you have an errno variable.  }
{ #undef HAVE_ERRNO }

{ Define if you have an environ variable.  }
{$define HAVE_ENVIRON 1}

{ Define if you have an __environ variable.  }
{ #undef HAVE___ENVIRON }

{ Define if you have a sys_siglist variable.  }
{$define HAVE_SYS_SIGLIST 1}

{ Define if you have a _sys_siglist variable.  }
{$define HAVE__SYS_SIGLIST 1}

{ Define if you have a _sys_nsig variable.  }
{ #undef HAVE__SYS_NSIG }

{ Define if you have a tzname variable.  }
{$define HAVE_TZNAME 1}

{ Define if you have a timezone variable.  }
{$define HAVE_TIMEZONE 1}

{ Define if you have a _timezone variable.  }
{ #undef HAVE__TIMEZONE }

{ Define if mkdir accepts two arguments.  }
{$define MKDIR_TWOARG 1}

{ Define if getpgrp takes an argument.  }
{ #undef GETPGRP_ARG }

{ Define if the errno variable is declared in <errno.h>.  }
{$define ERRNO_DECLARED 1}

{ Define if the environ variable is declared in <unistd.h>.  }
{ #undef ENVIRON_DECLARED }

{ Define if the __environ variable is declared in <unistd.h>.  }
{ #undef __ENVIRON_DECLARED }

{ Define if the _sys_siglist variable is declared in <signal.h> or <unistd.h>.  }
{ #undef _SYS_SIGLIST_DECLARED }

{ Define if the _sys_nsig variable is declared in <signal.h>.  }
{ #undef _SYS_NSIG_DECLARED }

{ Define if the tzname variable is declared in <time.h>.  }
{$define TZNAME_DECLARED 1}

{ Define if the timezone variable is declared in <time.h>.  }
{$define TIMEZONE_DECLARED 1}

{ Define if the _timezone variable is declared in <time.h>.  }
{ #undef _TIMEZONE_DECLARED }

{ Define if the sinl function is declared in <math.h>.  }
{$define SINL_DECLARED 1}

{ Define if the cosl function is declared in <math.h>.  }
{$define COSL_DECLARED 1}

{ Define if the sqrtl function is declared in <math.h>.  }
{$define SQRTL_DECLARED 1}

{ Define if the logl function is declared in <math.h>.  }
{$define LOGL_DECLARED 1}

{ Define if the expl function is declared in <math.h>.  }
{$define EXPL_DECLARED 1}

{ Define if the powl function is declared in <math.h>.  }
{$define POWL_DECLARED 1}

{ Define if the asinl function is declared in <math.h>.  }
{$define ASINL_DECLARED 1}

{ Define if the acosl function is declared in <math.h>.  }
{$define ACOSL_DECLARED 1}

{ Define if the atanl function is declared in <math.h>.  }
{$define ATANL_DECLARED 1}

{ Define if the frexpl function is declared in <math.h>.  }
{$define FREXPL_DECLARED 1}

{ Define if the isinfl function is declared in <math.h>.  }
{ #undef ISINFL_DECLARED }

{ Define if the isnanl function is declared in <math.h>.  }
{ #undef ISNANL_DECLARED }

{ Define if the sleep function is declared in <unistd.h>.  }
{$define SLEEP_DECLARED 1}

{ Define if the llseek function is declared in <unistd.h>.  }
{ #undef LLSEEK_DECLARED }

{ Define if the usleep function is declared in <unistd.h>.  }
{$define USLEEP_DECLARED 1}

{ Define if the setenv function is declared in <stdlib.h>.  }
{$define SETENV_DECLARED 1}

{ Define if the setregid function is declared in <unistd.h>.  }
{$define SETREGID_DECLARED 1}

{ Define if the setreuid function is declared in <unistd.h>.  }
{$define SETREUID_DECLARED 1}

{ Define if the unsetenv function is declared in <stdlib.h>.  }
{$define UNSETENV_DECLARED 1}

{ Define if the fstat64 function is declared in <sys/stat.h>.  }
{ #undef FSTAT64_DECLARED }

{ Define if the ftruncate64 function is declared in <unistd.h>.  }
{ #undef FTRUNCATE64_DECLARED }

{ Define if the lseek64 function is declared in <unistd.h>.  }
{ #undef LSEEK64_DECLARED }

{ Define if the lstat64 function is declared in <sys/stat.h>.  }
{ #undef LSTAT64_DECLARED }

{ Define if the open64 function is declared in <fcntl.h>.  }
{ #undef OPEN64_DECLARED }

{ Define if the stat64 function is declared in <sys/stat.h>.  }
{ #undef STAT64_DECLARED }

{ Define if the strsignal function is declared in <string.h> or <signal.h>.  }
{$define STRSIGNAL_DECLARED 1}

{ Define if __builtin_return_address supports a non-zero argument.  }
{$define HAVE_RETURN_ADDRESS_NON_ZERO 1}


{ Define to 1 if you have the `abort' function. }
{$define HAVE_ABORT 1}

{ Define to 1 if you have the `acosl' function. }
{$define HAVE_ACOSL 1}

{ Define to 1 if you have the `alarm' function. }
{$define HAVE_ALARM 1}

{ Define to 1 if you have the `asinl' function. }
{$define HAVE_ASINL 1}

{ Define to 1 if you have the `atanl' function. }
{$define HAVE_ATANL 1}

{ Define to 1 if you have the <bsd/signal.h> header file. }
{ #undef HAVE_BSD_SIGNAL_H }

{ Define to 1 if you have the `chdir' function. }
{$define HAVE_CHDIR 1}

{ Define to 1 if you have the `chmod' function. }
{$define HAVE_CHMOD 1}

{ Define to 1 if you have the `chown' function. }
{$define HAVE_CHOWN 1}

{ Define to 1 if you have the `cosl' function. }
{$define HAVE_COSL 1}

{ Define to 1 if you have the <crt_externs.h> header file. }
{ #undef HAVE_CRT_EXTERNS_H }

{ Define to 1 if you have the <ctype.h> header file. }
{$define HAVE_CTYPE_H 1}

{ Define to 1 if you have the declaration of `sys_siglist', and to 0 if you
   don't. }
{$define HAVE_DECL_SYS_SIGLIST 1}

{ Define to 1 if you have the <dirent.h> header file, and it defines `DIR'.
   }
{$define HAVE_DIRENT_H 1}

{ Define to 1 if you have the `dup2' function. }
{$define HAVE_DUP2 1}

{ Define to 1 if you have the `endpwent' function. }
{$define HAVE_ENDPWENT 1}

{ Define to 1 if you have the <errno.h> header file. }
{$define HAVE_ERRNO_H 1}

{ Define to 1 if you have the `executable_path' function. }
{ #undef HAVE_EXECUTABLE_PATH }

{ Define to 1 if you have the `expl' function. }
{$define HAVE_EXPL 1}

{ Define to 1 if you have the `fcntl' function. }
{$define HAVE_FCNTL 1}

{ Define to 1 if you have the <fcntl.h> header file. }
{$define HAVE_FCNTL_H 1}

{ Define to 1 if you have the `fnmatch' function. }
{$define HAVE_FNMATCH 1}

{ Define to 1 if you have the <fnmatch.h> header file. }
{$define HAVE_FNMATCH_H 1}

{ Define to 1 if you have the `frexpl' function. }
{$define HAVE_FREXPL 1}

{ Define to 1 if you have the `fstat' function. }
{$define HAVE_FSTAT 1}

{ Define to 1 if you have the `fstat64' function. }
{ #undef HAVE_FSTAT64 }

{ Define to 1 if you have the `fsync' function. }
{$define HAVE_FSYNC 1}

{ Define to 1 if you have the `ftruncate' function. }
{$define HAVE_FTRUNCATE 1}

{ Define to 1 if you have the `ftruncate64' function. }
{ #undef HAVE_FTRUNCATE64 }

{ Define to 1 if you have the `getcwd' function. }
{$define HAVE_GETCWD 1}

{ Define to 1 if you have the `getegid' function. }
{$define HAVE_GETEGID 1}

{ Define to 1 if you have the `getenv' function. }
{$define HAVE_GETENV 1}

{ Define to 1 if you have the `geteuid' function. }
{$define HAVE_GETEUID 1}

{ Define to 1 if you have the `getgid' function. }
{$define HAVE_GETGID 1}

{ Define to 1 if you have the `getpgid' function. }
{$define HAVE_GETPGID 1}

{ Define to 1 if you have the `getpgrp' function. }
{$define HAVE_GETPGRP 1}

{ Define to 1 if you have the `getpid' function. }
{$define HAVE_GETPID 1}

{ Define to 1 if you have the `getpwent' function. }
{$define HAVE_GETPWENT 1}

{ Define to 1 if you have the `getpwnam' function. }
{$define HAVE_GETPWNAM 1}

{ Define to 1 if you have the `getpwuid' function. }
{$define HAVE_GETPWUID 1}

{ Define to 1 if you have the `getrusage' function. }
{$define HAVE_GETRUSAGE 1}

{ Define to 1 if you have the `gettimeofday' function. }
{$define HAVE_GETTIMEOFDAY 1}

{ Define to 1 if you have the `gettmpdir' function. }
{ #undef HAVE_GETTMPDIR }

{ Define to 1 if you have the `getuid' function. }
{$define HAVE_GETUID 1}

{ Define to 1 if you have the `glob' function. }
{$define HAVE_GLOB 1}

{ Define to 1 if you have the <glob.h> header file. }
{$define HAVE_GLOB_H 1}

{ Define to 1 if you have the `hypot' function. }
{$define HAVE_HYPOT 1}

{ Define to 1 if you have the <inttypes.h> header file. }
{$define HAVE_INTTYPES_H 1}

{ Define to 1 if you have the `isatty' function. }
{$define HAVE_ISATTY 1}

{ Define to 1 if you have the `isinfl' function. }
{ #undef HAVE_ISINFL }

{ Define to 1 if you have the `isnanl' function. }
{ #undef HAVE_ISNANL }

{ Define to 1 if you have the `kill' function. }
{$define HAVE_KILL 1}

{ Define to 1 if you have the `iconv' library (-liconv). }
{$define HAVE_LIBICONV 1}

{ Define to 1 if you have the `intl' library (-lintl). }
{$define HAVE_LIBINTL 1}

{ Define to 1 if you have the `m' library (-lm). }
{$define HAVE_LIBM 1}

{ Define to 1 if you have the `os-hacks' library (-los-hacks). }
{ #undef HAVE_LIBOS_HACKS }

{ Define to 1 if you have the <limits.h> header file. }
{$define HAVE_LIMITS_H 1}

{ Define to 1 if you have the `llseek' function. }
{ #undef HAVE_LLSEEK }

{ Define to 1 if you have the <locale.h> header file. }
{$define HAVE_LOCALE_H 1}

{ Define to 1 if you have the `log1p' function. }
{$define HAVE_LOG1P 1}

{ Define to 1 if you have the `logl' function. }
{$define HAVE_LOGL 1}

{ Define to 1 if you have the `lseek' function. }
{$define HAVE_LSEEK 1}

{ Define to 1 if you have the `lseek64' function. }
{ #undef HAVE_LSEEK64 }

{ Define to 1 if you have the `lstat' function. }
{$define HAVE_LSTAT 1}

{ Define to 1 if you have the `lstat64' function. }
{ #undef HAVE_LSTAT64 }

{ Define to 1 if you have the `malloc_init' function. }
{ #undef HAVE_MALLOC_INIT }

{ Define to 1 if you have the <math.h> header file. }
{$define HAVE_MATH_H 1}

{ Define to 1 if you have the <memory.h> header file. }
{$define HAVE_MEMORY_H 1}

{ Define to 1 if you have the `mkdir' function. }
{$define HAVE_MKDIR 1}

{ Define to 1 if you have the `mmap' function. }
{$define HAVE_MMAP 1}

{ Define to 1 if you have the `munmap' function. }
{$define HAVE_MUNMAP 1}

{ Define to 1 if you have the <ndir.h> header file, and it defines `DIR'. }
{ #undef HAVE_NDIR_H }

{ Define to 1 if you have the `open64' function. }
{ #undef HAVE_OPEN64 }

{ Define to 1 if you have the <os-hacks.h> header file. }
{ #undef HAVE_OS_HACKS_H }

{ Define to 1 if you have the `powl' function. }
{$define HAVE_POWL 1}

{ Define to 1 if you have the <process.h> header file. }
{ #undef HAVE_PROCESS_H }

{ Define to 1 if you have the `putenv' function. }
{$define HAVE_PUTENV 1}

{ Define to 1 if you have the <pwd.h> header file. }
{$define HAVE_PWD_H 1}

{ Define to 1 if you have the `readlink' function. }
{$define HAVE_READLINK 1}

{ Define to 1 if you have the `realpath' function. }
{$define HAVE_REALPATH 1}

{ Define to 1 if you have the `rename' function. }
{$define HAVE_RENAME 1}

{ Define to 1 if you have the `rmdir' function. }
{$define HAVE_RMDIR 1}

{ Define to 1 if you have the `select' function. }
{$define HAVE_SELECT 1}

{ Define to 1 if you have the `setenv' function. }
{$define HAVE_SETENV 1}

{ Define to 1 if you have the `setgid' function. }
{$define HAVE_SETGID 1}

{ Define to 1 if you have the `setlocale' function. }
{$define HAVE_SETLOCALE 1}

{ Define to 1 if you have the `setpgid' function. }
{$define HAVE_SETPGID 1}

{ Define to 1 if you have the `setpwent' function. }
{$define HAVE_SETPWENT 1}

{ Define to 1 if you have the `setregid' function. }
{$define HAVE_SETREGID 1}

{ Define to 1 if you have the `setreuid' function. }
{$define HAVE_SETREUID 1}

{ Define to 1 if you have the `setuid' function. }
{$define HAVE_SETUID 1}

{ Define to 1 if you have the `sigaction' function. }
{$define HAVE_SIGACTION 1}

{ Define to 1 if you have the `sigblock' function. }
{$define HAVE_SIGBLOCK 1}

{ Define to 1 if you have the `siginterrupt' function. }
{$define HAVE_SIGINTERRUPT 1}

{ Define to 1 if you have the `signal' function. }
{$define HAVE_SIGNAL 1}

{ Define to 1 if you have the <signal.h> header file. }
{$define HAVE_SIGNAL_H 1}

{ Define to 1 if you have the `sigprocmask' function. }
{$define HAVE_SIGPROCMASK 1}

{ Define to 1 if you have the `sigsetmask' function. }
{$define HAVE_SIGSETMASK 1}

{ Define to 1 if you have the `sigvec' function. }
{$define HAVE_SIGVEC 1}

{ Define to 1 if you have the `sinl' function. }
{$define HAVE_SINL 1}

{ Define to 1 if you have the `sleep' function. }
{$define HAVE_SLEEP 1}

{ Define to 1 if you have the `sqrtl' function. }
{$define HAVE_SQRTL 1}

{ Define to 1 if you have the `stat' function. }
{$define HAVE_STAT 1}

{ Define to 1 if you have the `stat64' function. }
{ #undef HAVE_STAT64 }

{ Define to 1 if you have the `statfs' function. }
{$define HAVE_STATFS 1}

{ Define to 1 if you have the `statvfs' function. }
{$define HAVE_STATVFS 1}

{ Define to 1 if you have the <stdint.h> header file. }
{$define HAVE_STDINT_H 1}

{ Define to 1 if you have the <stdio.h> header file. }
{$define HAVE_STDIO_H 1}

{ Define to 1 if you have the <stdlib.h> header file. }
{$define HAVE_STDLIB_H 1}

{ Define to 1 if you have the `strerror' function. }
{$define HAVE_STRERROR 1}

{ Define to 1 if you have the `strftime' function. }
{$define HAVE_STRFTIME 1}

{ Define to 1 if you have the <strings.h> header file. }
{$define HAVE_STRINGS_H 1}

{ Define to 1 if you have the <string.h> header file. }
{$define HAVE_STRING_H 1}

{ Define to 1 if you have the `strsignal' function. }
{$define HAVE_STRSIGNAL 1}

{ Define to 1 if you have the <sys/dir.h> header file, and it defines `DIR'.
   }
{ #undef HAVE_SYS_DIR_H }

{ Define to 1 if you have the <sys/file.h> header file. }
{$define HAVE_SYS_FILE_H 1}

{ Define to 1 if you have the <sys/mman.h> header file. }
{$define HAVE_SYS_MMAN_H 1}

{ Define to 1 if you have the <sys/mount.h> header file. }
{$define HAVE_SYS_MOUNT_H 1}

{ Define to 1 if you have the <sys/ndir.h> header file, and it defines `DIR'.
   }
{ #undef HAVE_SYS_NDIR_H }

{ Define to 1 if you have the <sys/param.h> header file. }
{$define HAVE_SYS_PARAM_H 1}

{ Define to 1 if you have the <sys/resource.h> header file. }
{$define HAVE_SYS_RESOURCE_H 1}

{ Define to 1 if you have the <sys/select.h> header file. }
{$define HAVE_SYS_SELECT_H 1}

{ Define to 1 if you have the <sys/statvfs.h> header file. }
{$define HAVE_SYS_STATVFS_H 1}

{ Define to 1 if you have the <sys/stat.h> header file. }
{$define HAVE_SYS_STAT_H 1}

{ Define to 1 if you have the <sys/time.h> header file. }
{$define HAVE_SYS_TIME_H 1}

{ Define to 1 if you have the <sys/types.h> header file. }
{$define HAVE_SYS_TYPES_H 1}

{ Define to 1 if you have the <sys/utime.h> header file. }
{ #undef HAVE_SYS_UTIME_H }

{ Define to 1 if you have the <sys/utsname.h> header file. }
{$define HAVE_SYS_UTSNAME_H 1}

{ Define to 1 if you have the <sys/vfs.h> header file. }
{ #undef HAVE_SYS_VFS_H }

{ Define to 1 if you have <sys/wait.h> that is POSIX.1 compatible. }
{$define HAVE_SYS_WAIT_H 1}

{ Define to 1 if you have the `tcgetattr' function. }
{$define HAVE_TCGETATTR 1}

{ Define to 1 if you have the `tcgetpgrp' function. }
{$define HAVE_TCGETPGRP 1}

{ Define to 1 if you have the `tcsetattr' function. }
{$define HAVE_TCSETATTR 1}

{ Define to 1 if you have the `tcsetpgrp' function. }
{$define HAVE_TCSETPGRP 1}

{ Define to 1 if you have the <termios.h> header file. }
{$define HAVE_TERMIOS_H 1}

{ Define to 1 if you have the <termio.h> header file. }
{ #undef HAVE_TERMIO_H }

{ Define to 1 if you have the `time' function. }
{$define HAVE_TIME 1}

{ Define to 1 if you have the `ttyname' function. }
{$define HAVE_TTYNAME 1}

{ Define to 1 if you have the `umask' function. }
{$define HAVE_UMASK 1}

{ Define to 1 if you have the `uname' function. }
{$define HAVE_UNAME 1}

{ Define to 1 if you have the <unistd.h> header file. }
{$define HAVE_UNISTD_H 1}

{ Define to 1 if you have the `unlink' function. }
{$define HAVE_UNLINK 1}

{ Define to 1 if you have the `unsetenv' function. }
{$define HAVE_UNSETENV 1}

{ Define to 1 if you have the `usleep' function. }
{$define HAVE_USLEEP 1}

{ Define to 1 if you have the `utime' function. }
{$define HAVE_UTIME 1}

{ Define to 1 if you have the <utime.h> header file. }
{$define HAVE_UTIME_H 1}

{ Define to 1 if you have the `vfork' function. }
{$define HAVE_VFORK 1}

{ Define to 1 if you have the `waitpid' function. }
{$define HAVE_WAITPID 1}

{ Define to 1 if you have the `_sleep' function. }
{ #undef HAVE__SLEEP }

{ Define to the address where bug reports for this package should be sent. }
{$define PACKAGE_BUGREPORT ""}

{ Define to the full name of this package. }
{$define PACKAGE_NAME ""}

{ Define to the full name and version of this package. }
{$define PACKAGE_STRING ""}

{ Define to the one symbol short name of this package. }
{$define PACKAGE_TARNAME ""}

{ Define to the version of this package. }
{$define PACKAGE_VERSION ""}

{ Define to 1 if you have the ANSI C header files. }
{$define STDC_HEADERS 1}

{ Define to 1 if you can safely include both <sys/time.h> and <time.h>. }
{$define TIME_WITH_SYS_TIME 1}
